"""
Cloud AppStore - Modern PyCloud OS Uygulama Mağazası
Kategori bazlı gezinme, arama, kurulum/kaldırma, kullanıcı puanları
"""

import os
import sys
import json
import logging
import threading
from pathlib import Path
from typing import Dict, List, Optional, Any, Set
from datetime import datetime
from enum import Enum

try:
    from PyQt6.QtWidgets import *
    from PyQt6.QtCore import *
    from PyQt6.QtGui import *
    from PyQt6.QtNetwork import *
except ImportError:
    print("PyQt6 not available for Cloud AppStore")
    sys.exit(1)

class AppStatus(Enum):
    """Uygulama durumları"""
    AVAILABLE = "available"
    INSTALLED = "installed"
    UPDATING = "updating"
    INSTALLING = "installing"
    REMOVING = "removing"
    ERROR = "error"

class ViewMode(Enum):
    """Görünüm modları"""
    GRID = "grid"
    LIST = "list"
    DETAILED = "detailed"

class SortMode(Enum):
    """Sıralama modları"""
    NAME = "name"
    CATEGORY = "category"
    DEVELOPER = "developer"
    RATING = "rating"
    DOWNLOADS = "downloads"
    DATE = "date"

class AppInfo:
    """Uygulama bilgi sınıfı"""
    
    def __init__(self, data: Dict[str, Any]):
        self.app_id = data.get("app_id", "")
        self.name = data.get("name", "")
        self.version = data.get("version", "1.0.0")
        self.description = data.get("description", "")
        self.category = data.get("category", "Diğer")
        self.developer = data.get("developer", "Bilinmeyen")
        self.icon_path = data.get("icon_path", "")
        self.app_path = data.get("app_path", "")
        self.tags = data.get("tags", [])
        self.screenshots = data.get("screenshots", [])
        self.homepage = data.get("homepage", "")
        self.license = data.get("license", "")
        self.requires = data.get("requires", [])
        self.permissions = data.get("permissions", [])
        
        # Ek bilgiler
        self.rating = data.get("rating", 0.0)
        self.downloads = data.get("downloads", 0)
        self.reviews = data.get("reviews", [])
        self.last_updated = data.get("last_updated", "")
        self.size = data.get("size", 0)
        
        # Durum
        self.status = AppStatus.AVAILABLE
        self.is_installed = False
        self.installed_version = ""
        
    def has_update(self) -> bool:
        """Güncelleme var mı?"""
        if not self.is_installed:
            return False
        try:
            from packaging import version
            return version.parse(self.version) > version.parse(self.installed_version)
        except:
            return self.version != self.installed_version

class AppDataManager:
    """Uygulama veri yöneticisi"""
    
    def __init__(self, kernel=None):
        self.kernel = kernel
        self.logger = logging.getLogger("AppDataManager")
        self.apps: Dict[str, AppInfo] = {}
        self.installed_apps: Set[str] = set()
        
        # AppExplorer entegrasyonu
        self.app_explorer = None
        if kernel:
            self.app_explorer = kernel.get_module("appexplorer")
        
        # Clapp entegrasyonu
        self.clapp_core = None
        self.clapp_repo = None
        if kernel:
            self.clapp_core = kernel.get_module("clapp_core")
            self.clapp_repo = kernel.get_module("clapp_repo")
    
    def load_apps(self) -> List[AppInfo]:
        """Uygulamaları yükle"""
        apps = []
        
        try:
            # AppExplorer'dan yüklü uygulamaları al
            if self.app_explorer:
                discovered_apps = self.app_explorer.get_all_apps()
                
                for discovered_app in discovered_apps:
                    app_data = {
                        "app_id": discovered_app.app_id,
                        "name": discovered_app.name,
                        "version": discovered_app.version,
                        "description": discovered_app.description,
                        "category": discovered_app.category,
                        "developer": discovered_app.developer,
                        "icon_path": discovered_app.icon_path,
                        "app_path": discovered_app.app_path,
                        "tags": discovered_app.tags,
                        "rating": 4.0,  # Varsayılan rating
                        "downloads": 100,  # Varsayılan indirme sayısı
                        "last_updated": discovered_app.last_validated
                    }
                    
                    app_info = AppInfo(app_data)
                    app_info.is_installed = True
                    app_info.installed_version = app_info.version
                    app_info.status = AppStatus.INSTALLED
                    
                    apps.append(app_info)
                    self.apps[app_info.app_id] = app_info
                    self.installed_apps.add(app_info.app_id)
                
                self.logger.info(f"Loaded {len(apps)} installed apps from AppExplorer")
            
            # Clapp Repository'den mevcut uygulamaları ekle
            if self.clapp_repo:
                repo_apps = self._load_clapp_repository_apps()
                for app_info in repo_apps:
                    if app_info.app_id not in self.apps:
                        apps.append(app_info)
                        self.apps[app_info.app_id] = app_info
                    else:
                        # Güncelleme kontrolü
                        existing_app = self.apps[app_info.app_id]
                        if app_info.version != existing_app.version:
                            existing_app.version = app_info.version
            else:
                # Fallback: Mock veriler
                repo_apps = self._load_repository_apps()
                for app_info in repo_apps:
                    if app_info.app_id not in self.apps:
                        apps.append(app_info)
                        self.apps[app_info.app_id] = app_info
                        
        except Exception as e:
            self.logger.error(f"Failed to load apps: {e}")
        
        return apps
    
    def _load_clapp_repository_apps(self) -> List[AppInfo]:
        """Clapp Repository'den uygulamaları yükle"""
        apps = []
        
        try:
            if not self.clapp_repo:
                return apps
            
            # Tüm repository'leri yenile
            self.clapp_repo.refresh_repositories()
            
            # Tüm paketleri al
            packages = self.clapp_repo.get_all_packages()
            
            for package in packages:
                app_data = {
                    "app_id": package.id,
                    "name": package.name,
                    "version": package.version,
                    "description": package.description,
                    "category": package.category,
                    "developer": package.developer,
                    "icon_path": "",  # Repository'den ikon URL'si gelecek
                    "app_path": "",
                    "tags": getattr(package, 'tags', []),
                    "rating": getattr(package, 'rating', 4.0),
                    "downloads": getattr(package, 'downloads', 0),
                    "last_updated": getattr(package, 'last_updated', ""),
                    "homepage": getattr(package, 'homepage', ""),
                    "license": getattr(package, 'license', ""),
                    "requires": getattr(package, 'requires', []),
                    "permissions": getattr(package, 'permissions', []),
                    "screenshots": getattr(package, 'screenshots', []),
                    "size": getattr(package, 'size', 0)
                }
                
                app_info = AppInfo(app_data)
                apps.append(app_info)
                
            self.logger.info(f"Loaded {len(apps)} apps from Clapp repositories")
            
        except Exception as e:
            self.logger.error(f"Failed to load Clapp repository apps: {e}")
        
        return apps
    
    def _load_repository_apps(self) -> List[AppInfo]:
        """Repository'den uygulamaları yükle (simülasyon)"""
        # Simülasyon verisi - gerçek uygulamada repository'den gelecek
        mock_apps = [
            {
                "app_id": "cloud_calculator",
                "name": "Cloud Calculator",
                "version": "1.2.0",
                "description": "Modern hesap makinesi uygulaması. Temel matematik işlemleri, bilimsel hesaplamalar ve geçmiş desteği.",
                "category": "Araçlar",
                "developer": "PyCloud Team",
                "icon_path": "",
                "app_path": "",
                "tags": ["calculator", "math", "utility"],
                "rating": 4.5,
                "downloads": 1250,
                "last_updated": "2024-01-15"
            },
            {
                "app_id": "cloud_music",
                "name": "Cloud Music Player",
                "version": "2.1.0",
                "description": "Güçlü müzik çalar. MP3, FLAC, OGG desteği. Playlist yönetimi ve equalizer.",
                "category": "Multimedya",
                "developer": "AudioSoft",
                "icon_path": "",
                "app_path": "",
                "tags": ["music", "player", "audio"],
                "rating": 4.8,
                "downloads": 3400,
                "last_updated": "2024-01-20"
            },
            {
                "app_id": "cloud_paint",
                "name": "Cloud Paint",
                "version": "1.5.0",
                "description": "Basit çizim ve boyama uygulaması. Fırça araçları, katmanlar ve filtreler.",
                "category": "Grafik",
                "developer": "ArtStudio",
                "icon_path": "",
                "app_path": "",
                "tags": ["paint", "drawing", "graphics"],
                "rating": 4.2,
                "downloads": 890,
                "last_updated": "2024-01-10"
            },
            {
                "app_id": "cloud_chess",
                "name": "Cloud Chess",
                "version": "1.0.0",
                "description": "Klasik satranç oyunu. AI rakip, çoklu oyuncu desteği ve turnuva modu.",
                "category": "Oyunlar",
                "developer": "GameDev Studio",
                "icon_path": "",
                "app_path": "",
                "tags": ["chess", "game", "strategy"],
                "rating": 4.6,
                "downloads": 2100,
                "last_updated": "2024-01-05"
            },
            {
                "app_id": "cloud_markdown",
                "name": "Markdown Editor",
                "version": "1.3.0",
                "description": "Gelişmiş Markdown editörü. Canlı önizleme, syntax highlighting ve export seçenekleri.",
                "category": "Geliştirme",
                "developer": "DevTools Inc",
                "icon_path": "",
                "app_path": "",
                "tags": ["markdown", "editor", "development"],
                "rating": 4.4,
                "downloads": 1680,
                "last_updated": "2024-01-18"
            }
        ]
        
        apps = []
        for app_data in mock_apps:
            app_info = AppInfo(app_data)
            apps.append(app_info)
        
        return apps
    
    def get_apps_by_category(self, category: str) -> List[AppInfo]:
        """Kategoriye göre uygulamaları getir"""
        if category == "Tümü":
            return list(self.apps.values())
        elif category == "Yüklü":
            return [app for app in self.apps.values() if app.is_installed]
        elif category == "Güncellemeler":
            return [app for app in self.apps.values() if app.has_update()]
        else:
            return [app for app in self.apps.values() if app.category == category]
    
    def search_apps(self, query: str) -> List[AppInfo]:
        """Uygulama ara"""
        if not query:
            return list(self.apps.values())
        
        query = query.lower()
        results = []
        
        for app in self.apps.values():
            # Ad, açıklama, geliştirici ve etiketlerde ara
            if (query in app.name.lower() or 
                query in app.description.lower() or
                query in app.developer.lower() or
                any(query in tag.lower() for tag in app.tags)):
                results.append(app)
        
        return results
    
    def get_category_counts(self) -> Dict[str, int]:
        """Kategori sayılarını getir"""
        counts = {}
        
        # Tüm uygulamalar
        counts["Tümü"] = len(self.apps)
        
        # Yüklü uygulamalar
        counts["Yüklü"] = len([app for app in self.apps.values() if app.is_installed])
        
        # Güncellemeler
        counts["Güncellemeler"] = len([app for app in self.apps.values() if app.has_update()])
        
        # Kategoriler
        for app in self.apps.values():
            if app.category not in counts:
                counts[app.category] = 0
            counts[app.category] += 1
        
        return counts
    
    def install_app(self, app_id: str) -> bool:
        """Uygulama kur"""
        if app_id not in self.apps:
            return False
        
        app = self.apps[app_id]
        app.status = AppStatus.INSTALLING
        
        try:
            # Clapp Core kullanarak kurulum yap
            if self.clapp_core:
                result, message = self.clapp_core._cmd_install([app_id])
                
                if result.name == "SUCCESS":
                    app.is_installed = True
                    app.installed_version = app.version
                    app.status = AppStatus.INSTALLED
                    self.installed_apps.add(app_id)
                    
                    self.logger.info(f"App installed via Clapp: {app.name}")
                    return True
                else:
                    app.status = AppStatus.ERROR
                    self.logger.error(f"Clapp install failed: {message}")
                    return False
            
            # Fallback: AppKit kullanarak kurulum yap
            elif self.kernel:
                appkit = self.kernel.get_module("appkit")
                if appkit:
                    # Gerçek kurulum işlemi burada olacak
                    pass
            
            # Simülasyon için başarılı kabul et
            app.is_installed = True
            app.installed_version = app.version
            app.status = AppStatus.INSTALLED
            self.installed_apps.add(app_id)
            
            self.logger.info(f"App installed (fallback): {app.name}")
            return True
            
        except Exception as e:
            app.status = AppStatus.ERROR
            self.logger.error(f"Failed to install app {app_id}: {e}")
            return False
    
    def remove_app(self, app_id: str) -> bool:
        """Uygulama kaldır"""
        if app_id not in self.apps or app_id not in self.installed_apps:
            return False
        
        app = self.apps[app_id]
        app.status = AppStatus.REMOVING
        
        try:
            # Clapp Core kullanarak kaldırma yap
            if self.clapp_core:
                result, message = self.clapp_core._cmd_remove([app_id])
                
                if result.name == "SUCCESS":
                    app.is_installed = False
                    app.installed_version = ""
                    app.status = AppStatus.AVAILABLE
                    self.installed_apps.discard(app_id)
                    
                    self.logger.info(f"App removed via Clapp: {app.name}")
                    return True
                else:
                    app.status = AppStatus.ERROR
                    self.logger.error(f"Clapp remove failed: {message}")
                    return False
            
            # Fallback: AppExplorer'dan kaldır
            elif self.app_explorer:
                self.app_explorer.indexer.remove_app(app_id)
            
            # AppKit kullanarak kaldırma işlemi
            elif self.kernel:
                appkit = self.kernel.get_module("appkit")
                if appkit:
                    # Gerçek kaldırma işlemi burada olacak
                    pass
            
            # Simülasyon için başarılı kabul et
            app.is_installed = False
            app.installed_version = ""
            app.status = AppStatus.AVAILABLE
            self.installed_apps.discard(app_id)
            
            self.logger.info(f"App removed (fallback): {app.name}")
            return True
            
        except Exception as e:
            app.status = AppStatus.ERROR
            self.logger.error(f"Failed to remove app {app_id}: {e}")
            return False
    
    def update_app(self, app_id: str) -> bool:
        """Uygulama güncelle"""
        if app_id not in self.apps or not self.apps[app_id].has_update():
            return False
        
        app = self.apps[app_id]
        app.status = AppStatus.UPDATING
        
        try:
            # Clapp Core kullanarak güncelleme yap
            if self.clapp_core:
                result, message = self.clapp_core._cmd_update([app_id])
                
                if result.name == "SUCCESS":
                    app.installed_version = app.version
                    app.status = AppStatus.INSTALLED
                    
                    self.logger.info(f"App updated via Clapp: {app.name}")
                    return True
                else:
                    app.status = AppStatus.ERROR
                    self.logger.error(f"Clapp update failed: {message}")
                    return False
            
            # Fallback: Güncelleme işlemi (simülasyon)
            app.installed_version = app.version
            app.status = AppStatus.INSTALLED
            
            self.logger.info(f"App updated (fallback): {app.name}")
            return True
            
        except Exception as e:
            app.status = AppStatus.ERROR
            self.logger.error(f"Failed to update app {app_id}: {e}")
            return False

class CloudAppStore(QMainWindow):
    """Modern Cloud AppStore ana sınıfı"""
    
    def __init__(self, kernel=None):
        super().__init__()
        self.kernel = kernel
        self.logger = logging.getLogger("CloudAppStore")
        
        # Veri yöneticisi
        self.data_manager = AppDataManager(kernel)
        
        # UI durumu
        self.current_apps: List[AppInfo] = []
        self.filtered_apps: List[AppInfo] = []
        self.current_category = "Tümü"
        self.current_search = ""
        self.current_view_mode = ViewMode.GRID
        self.current_sort_mode = SortMode.NAME
        
        # Tema
        self.dark_mode = self.detect_dark_mode()
        
        # UI kurulumu
        self.setup_ui()
        self.setup_connections()
        self.apply_theme()
        
        # Veri yükleme
        self.load_data()
        
        self.logger.info("Cloud AppStore initialized")
    
    def detect_dark_mode(self) -> bool:
        """Dark mode algıla"""
        try:
            if self.kernel:
                config = self.kernel.get_module("config")
                if config:
                    theme_config = config.get("theme", {})
                    return theme_config.get("dark_mode", False)
            
            # Fallback
            palette = QApplication.palette()
            window_color = palette.color(QPalette.ColorRole.Window)
            return window_color.lightness() < 128
            
        except Exception:
            return False
    
    def setup_ui(self):
        """UI kurulumu"""
        self.setWindowTitle("Cloud AppStore - Modern Uygulama Mağazası")
        self.setGeometry(100, 100, 1200, 800)
        
        # Ana widget
        central_widget = QWidget()
        self.setCentralWidget(central_widget)
        
        # Ana layout
        main_layout = QHBoxLayout(central_widget)
        main_layout.setContentsMargins(0, 0, 0, 0)
        main_layout.setSpacing(0)
        
        # Sol panel (kategoriler)
        from .widgets import CategorySidebar
        self.sidebar = CategorySidebar(self.dark_mode)
        self.sidebar.setFixedWidth(250)
        main_layout.addWidget(self.sidebar)
        
        # Sağ panel (ana içerik)
        right_panel = QWidget()
        right_layout = QVBoxLayout(right_panel)
        right_layout.setContentsMargins(0, 0, 0, 0)
        right_layout.setSpacing(0)
        
        # Üst bar (arama ve filtreler)
        from .widgets import SearchBar
        self.search_bar = SearchBar(self.dark_mode)
        right_layout.addWidget(self.search_bar)
        
        # Toolbar
        self.setup_toolbar()
        right_layout.addWidget(self.toolbar)
        
        # Uygulama listesi (scroll area)
        self.setup_app_area()
        right_layout.addWidget(self.app_scroll_area, 1)
        
        # Alt bar (durum)
        self.setup_status_bar()
        right_layout.addWidget(self.status_bar)
        
        main_layout.addWidget(right_panel, 1)
    
    def setup_toolbar(self):
        """Toolbar kurulumu"""
        self.toolbar = QWidget()
        toolbar_layout = QHBoxLayout(self.toolbar)
        toolbar_layout.setContentsMargins(12, 8, 12, 8)
        toolbar_layout.setSpacing(8)
        
        # Sol taraf: Görünüm butonları
        view_group = QButtonGroup(self)
        
        self.grid_view_btn = QPushButton("⊞ Izgara")
        self.grid_view_btn.setCheckable(True)
        self.grid_view_btn.setChecked(True)
        view_group.addButton(self.grid_view_btn, 0)
        
        self.list_view_btn = QPushButton("☰ Liste")
        self.list_view_btn.setCheckable(True)
        view_group.addButton(self.list_view_btn, 1)
        
        self.detailed_view_btn = QPushButton("📋 Detaylı")
        self.detailed_view_btn.setCheckable(True)
        view_group.addButton(self.detailed_view_btn, 2)
        
        toolbar_layout.addWidget(self.grid_view_btn)
        toolbar_layout.addWidget(self.list_view_btn)
        toolbar_layout.addWidget(self.detailed_view_btn)
        
        toolbar_layout.addWidget(QFrame())  # Ayırıcı
        
        # Orta: Sıralama
        toolbar_layout.addWidget(QLabel("Sırala:"))
        
        self.sort_combo = QComboBox()
        self.sort_combo.addItems([
            "Ada göre",
            "Kategoriye göre", 
            "Geliştiriciye göre",
            "Puana göre",
            "İndirme sayısına göre"
        ])
        toolbar_layout.addWidget(self.sort_combo)
        
        toolbar_layout.addStretch()
        
        # Sağ taraf: Yenile ve ayarlar
        self.refresh_btn = QPushButton("🔄 Yenile")
        toolbar_layout.addWidget(self.refresh_btn)
        
        self.settings_btn = QPushButton("⚙️")
        self.settings_btn.setFixedSize(32, 32)
        toolbar_layout.addWidget(self.settings_btn)
    
    def setup_app_area(self):
        """Uygulama alanı kurulumu"""
        self.app_scroll_area = QScrollArea()
        self.app_scroll_area.setWidgetResizable(True)
        self.app_scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
        
        # İçerik widget'ı
        self.app_content_widget = QWidget()
        self.app_scroll_area.setWidget(self.app_content_widget)
        
        # Layout (grid olarak başla)
        self.setup_grid_layout()
    
    def setup_grid_layout(self):
        """Grid layout kurulumu"""
        # Eski layout'u temizle
        if self.app_content_widget.layout():
            QWidget().setLayout(self.app_content_widget.layout())
        
        # Grid layout
        self.app_layout = QGridLayout(self.app_content_widget)
        self.app_layout.setContentsMargins(12, 12, 12, 12)
        self.app_layout.setSpacing(12)
        
        # Responsive grid (pencere boyutuna göre sütun sayısı)
        self.update_grid_columns()
    
    def setup_list_layout(self):
        """Liste layout kurulumu"""
        # Eski layout'u temizle
        if self.app_content_widget.layout():
            QWidget().setLayout(self.app_content_widget.layout())
        
        # Vertical layout
        self.app_layout = QVBoxLayout(self.app_content_widget)
        self.app_layout.setContentsMargins(12, 12, 12, 12)
        self.app_layout.setSpacing(8)
        self.app_layout.addStretch()  # Alt boşluk
    
    def setup_status_bar(self):
        """Durum çubuğu kurulumu"""
        self.status_bar = QWidget()
        status_layout = QHBoxLayout(self.status_bar)
        status_layout.setContentsMargins(12, 8, 12, 8)
        
        self.status_label = QLabel("Hazır")
        status_layout.addWidget(self.status_label)
        
        status_layout.addStretch()
        
        self.app_count_label = QLabel("")
        status_layout.addWidget(self.app_count_label)
    
    def setup_connections(self):
        """Sinyal bağlantıları"""
        # Sidebar
        self.sidebar.category_selected.connect(self.on_category_changed)
        
        # Search bar
        self.search_bar.search_requested.connect(self.on_search_requested)
        self.search_bar.filter_changed.connect(self.on_filter_changed)
        
        # Toolbar
        self.grid_view_btn.clicked.connect(lambda: self.change_view_mode(ViewMode.GRID))
        self.list_view_btn.clicked.connect(lambda: self.change_view_mode(ViewMode.LIST))
        self.detailed_view_btn.clicked.connect(lambda: self.change_view_mode(ViewMode.DETAILED))
        
        self.sort_combo.currentTextChanged.connect(self.on_sort_changed)
        self.refresh_btn.clicked.connect(self.refresh_data)
        self.settings_btn.clicked.connect(self.show_settings)
    
    def apply_theme(self):
        """Tema uygula"""
        if self.dark_mode:
            self.apply_dark_theme()
        else:
            self.apply_light_theme()
    
    def apply_dark_theme(self):
        """Koyu tema"""
        self.setStyleSheet("""
            QMainWindow {
                background-color: #1e1e1e;
                color: #ffffff;
            }
            QWidget {
                background-color: #1e1e1e;
                color: #ffffff;
            }
            QScrollArea {
                border: none;
                background-color: #2d2d2d;
            }
            QPushButton {
                background-color: #404040;
                border: 1px solid #555;
                border-radius: 6px;
                padding: 6px 12px;
                color: #ffffff;
            }
            QPushButton:hover {
                background-color: #505050;
            }
            QPushButton:checked {
                background-color: #2196f3;
            }
            QComboBox {
                background-color: #404040;
                border: 1px solid #555;
                border-radius: 6px;
                padding: 6px;
                color: #ffffff;
            }
            QLabel {
                color: #ffffff;
            }
        """)
    
    def apply_light_theme(self):
        """Açık tema"""
        self.setStyleSheet("""
            QMainWindow {
                background-color: #f5f5f5;
                color: #333;
            }
            QWidget {
                background-color: #f5f5f5;
                color: #333;
            }
            QScrollArea {
                border: none;
                background-color: white;
            }
            QPushButton {
                background-color: #ffffff;
                border: 1px solid #ddd;
                border-radius: 6px;
                padding: 6px 12px;
                color: #333;
            }
            QPushButton:hover {
                background-color: #f0f0f0;
            }
            QPushButton:checked {
                background-color: #2196f3;
                color: white;
            }
            QComboBox {
                background-color: white;
                border: 1px solid #ddd;
                border-radius: 6px;
                padding: 6px;
                color: #333;
            }
            QLabel {
                color: #333;
            }
        """)
    
    def load_data(self):
        """Veri yükle"""
        try:
            self.current_apps = self.data_manager.load_apps()
            self.update_category_counts()
            self.filter_and_display_apps()
            
            self.status_label.setText(f"{len(self.current_apps)} uygulama yüklendi")
            
        except Exception as e:
            self.logger.error(f"Failed to load data: {e}")
            self.status_label.setText("Veri yükleme hatası")
    
    def refresh_data(self):
        """Veriyi yenile"""
        self.status_label.setText("Yenileniyor...")
        
        # Thread'de yenile
        def refresh_worker():
            try:
                self.current_apps = self.data_manager.load_apps()
                QTimer.singleShot(0, self.on_refresh_completed)
            except Exception as e:
                self.logger.error(f"Refresh failed: {e}")
                QTimer.singleShot(0, lambda: self.status_label.setText("Yenileme hatası"))
        
        thread = threading.Thread(target=refresh_worker)
        thread.daemon = True
        thread.start()
    
    def on_refresh_completed(self):
        """Yenileme tamamlandı"""
        self.update_category_counts()
        self.filter_and_display_apps()
        self.status_label.setText("Yenileme tamamlandı")
    
    def update_category_counts(self):
        """Kategori sayılarını güncelle"""
        counts = self.data_manager.get_category_counts()
        self.sidebar.update_category_counts(counts)
    
    def filter_and_display_apps(self):
        """Uygulamaları filtrele ve göster"""
        # Kategori filtresi
        if self.current_category:
            self.filtered_apps = self.data_manager.get_apps_by_category(self.current_category)
        else:
            self.filtered_apps = self.current_apps.copy()
        
        # Arama filtresi
        if self.current_search:
            self.filtered_apps = [app for app in self.filtered_apps 
                                if self.current_search.lower() in app.name.lower() or
                                   self.current_search.lower() in app.description.lower()]
        
        # Sıralama
        self.sort_apps()
        
        # Görüntüle
        self.display_apps()
        
        # Durum güncelle
        self.app_count_label.setText(f"{len(self.filtered_apps)} uygulama")
    
    def sort_apps(self):
        """Uygulamaları sırala"""
        if self.current_sort_mode == SortMode.NAME:
            self.filtered_apps.sort(key=lambda app: app.name.lower())
        elif self.current_sort_mode == SortMode.CATEGORY:
            self.filtered_apps.sort(key=lambda app: app.category)
        elif self.current_sort_mode == SortMode.DEVELOPER:
            self.filtered_apps.sort(key=lambda app: app.developer)
        elif self.current_sort_mode == SortMode.RATING:
            self.filtered_apps.sort(key=lambda app: app.rating, reverse=True)
        elif self.current_sort_mode == SortMode.DOWNLOADS:
            self.filtered_apps.sort(key=lambda app: app.downloads, reverse=True)
    
    def display_apps(self):
        """Uygulamaları görüntüle"""
        # Mevcut widget'ları temizle
        self.clear_app_layout()
        
        if not self.filtered_apps:
            # Boş durum
            empty_label = QLabel("Hiç uygulama bulunamadı")
            empty_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
            empty_label.setStyleSheet("font-size: 16px; color: #666; margin: 50px;")
            
            if self.current_view_mode == ViewMode.GRID:
                self.app_layout.addWidget(empty_label, 0, 0)
            else:
                self.app_layout.addWidget(empty_label)
            return
        
        # Uygulama kartlarını oluştur
        from .widgets import ModernAppCard
        
        for i, app_info in enumerate(self.filtered_apps):
            card = ModernAppCard(app_info, self.current_view_mode, self.dark_mode)
            
            # Sinyal bağlantıları
            card.install_requested.connect(self.install_app)
            card.remove_requested.connect(self.remove_app)
            card.update_requested.connect(self.update_app)
            card.info_requested.connect(self.show_app_info)
            
            # Layout'a ekle
            if self.current_view_mode == ViewMode.GRID:
                row = i // self.grid_columns
                col = i % self.grid_columns
                self.app_layout.addWidget(card, row, col)
            else:
                self.app_layout.addWidget(card)
    
    def clear_app_layout(self):
        """Uygulama layout'unu temizle"""
        while self.app_layout.count():
            child = self.app_layout.takeAt(0)
            if child.widget():
                child.widget().deleteLater()
    
    def update_grid_columns(self):
        """Grid sütun sayısını güncelle"""
        width = self.app_scroll_area.width() - 50  # Scroll bar için boşluk
        card_width = 220  # Kart genişliği + margin
        self.grid_columns = max(1, width // card_width)
    
    def change_view_mode(self, mode: ViewMode):
        """Görünüm modunu değiştir"""
        self.current_view_mode = mode
        
        if mode == ViewMode.GRID:
            self.setup_grid_layout()
        else:
            self.setup_list_layout()
        
        self.display_apps()
    
    def on_category_changed(self, category: str):
        """Kategori değişti"""
        self.current_category = category
        self.filter_and_display_apps()
    
    def on_search_requested(self, query: str):
        """Arama yapıldı"""
        self.current_search = query
        self.filter_and_display_apps()
    
    def on_filter_changed(self, filters: Dict[str, Any]):
        """Filtre değişti"""
        if "sort" in filters:
            sort_map = {
                "name": SortMode.NAME,
                "category": SortMode.CATEGORY,
                "developer": SortMode.DEVELOPER,
                "rating": SortMode.RATING,
                "downloads": SortMode.DOWNLOADS
            }
            self.current_sort_mode = sort_map.get(filters["sort"], SortMode.NAME)
            self.filter_and_display_apps()
        
        if "view" in filters:
            view_map = {
                "grid": ViewMode.GRID,
                "list": ViewMode.LIST,
                "detailed": ViewMode.DETAILED
            }
            mode = view_map.get(filters["view"], ViewMode.GRID)
            self.change_view_mode(mode)
    
    def on_sort_changed(self, sort_text: str):
        """Sıralama değişti"""
        sort_map = {
            "Ada göre": SortMode.NAME,
            "Kategoriye göre": SortMode.CATEGORY,
            "Geliştiriciye göre": SortMode.DEVELOPER,
            "Puana göre": SortMode.RATING,
            "İndirme sayısına göre": SortMode.DOWNLOADS
        }
        self.current_sort_mode = sort_map.get(sort_text, SortMode.NAME)
        self.filter_and_display_apps()
    
    def install_app(self, app_id: str):
        """Uygulama kur"""
        self.status_label.setText(f"Kuruluyor: {app_id}")
        
        # Thread'de kur
        def install_worker():
            success = self.data_manager.install_app(app_id)
            QTimer.singleShot(0, lambda: self.on_install_completed(app_id, success))
        
        thread = threading.Thread(target=install_worker)
        thread.daemon = True
        thread.start()
    
    def remove_app(self, app_id: str):
        """Uygulama kaldır"""
        # Onay al
        app = self.data_manager.apps.get(app_id)
        if not app:
            return
        
        reply = QMessageBox.question(
            self, "Uygulama Kaldır",
            f"'{app.name}' uygulamasını kaldırmak istediğinizden emin misiniz?",
            QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No,
            QMessageBox.StandardButton.No
        )
        
        if reply == QMessageBox.StandardButton.Yes:
            self.status_label.setText(f"Kaldırılıyor: {app.name}")
            
            # Thread'de kaldır
            def remove_worker():
                success = self.data_manager.remove_app(app_id)
                QTimer.singleShot(0, lambda: self.on_remove_completed(app_id, success))
            
            thread = threading.Thread(target=remove_worker)
            thread.daemon = True
            thread.start()
    
    def update_app(self, app_id: str):
        """Uygulama güncelle"""
        self.status_label.setText(f"Güncelleniyor: {app_id}")
        
        # Thread'de güncelle
        def update_worker():
            success = self.data_manager.update_app(app_id)
            QTimer.singleShot(0, lambda: self.on_update_completed(app_id, success))
        
        thread = threading.Thread(target=update_worker)
        thread.daemon = True
        thread.start()
    
    def on_install_completed(self, app_id: str, success: bool):
        """Kurulum tamamlandı"""
        if success:
            self.status_label.setText("Kurulum başarılı")
            self.update_category_counts()
            self.filter_and_display_apps()
        else:
            self.status_label.setText("Kurulum başarısız")
    
    def on_remove_completed(self, app_id: str, success: bool):
        """Kaldırma tamamlandı"""
        if success:
            self.status_label.setText("Kaldırma başarılı")
            self.update_category_counts()
            self.filter_and_display_apps()
        else:
            self.status_label.setText("Kaldırma başarısız")
    
    def on_update_completed(self, app_id: str, success: bool):
        """Güncelleme tamamlandı"""
        if success:
            self.status_label.setText("Güncelleme başarılı")
            self.filter_and_display_apps()
        else:
            self.status_label.setText("Güncelleme başarısız")
    
    def show_app_info(self, app_id: str):
        """Uygulama bilgilerini göster"""
        app = self.data_manager.apps.get(app_id)
        if not app:
            return
        
        # Detay dialog'u göster
        from .dialogs import AppDetailDialog
        dialog = AppDetailDialog(app, self.dark_mode, self)
        
        # Sinyal bağlantıları
        dialog.install_requested.connect(self.install_app)
        dialog.remove_requested.connect(self.remove_app)
        dialog.update_requested.connect(self.update_app)
        
        dialog.exec()
    
    def show_settings(self):
        """Ayarları göster"""
        # Ayar dialog'u
        dialog = QDialog(self)
        dialog.setWindowTitle("AppStore Ayarları")
        dialog.setFixedSize(400, 300)
        
        layout = QVBoxLayout(dialog)
        
        # Tema seçimi
        theme_group = QGroupBox("Tema")
        theme_layout = QVBoxLayout(theme_group)
        
        light_radio = QRadioButton("Açık Tema")
        dark_radio = QRadioButton("Koyu Tema")
        
        if self.dark_mode:
            dark_radio.setChecked(True)
        else:
            light_radio.setChecked(True)
        
        theme_layout.addWidget(light_radio)
        theme_layout.addWidget(dark_radio)
        layout.addWidget(theme_group)
        
        # Butonlar
        button_layout = QHBoxLayout()
        
        ok_btn = QPushButton("Tamam")
        cancel_btn = QPushButton("İptal")
        
        button_layout.addStretch()
        button_layout.addWidget(ok_btn)
        button_layout.addWidget(cancel_btn)
        
        layout.addLayout(button_layout)
        
        # Bağlantılar
        ok_btn.clicked.connect(dialog.accept)
        cancel_btn.clicked.connect(dialog.reject)
        
        # Tema değişikliği
        def on_theme_changed():
            self.dark_mode = dark_radio.isChecked()
            self.apply_theme()
            self.sidebar.dark_mode = self.dark_mode
            self.sidebar.apply_theme()
            self.search_bar.dark_mode = self.dark_mode
            self.search_bar.apply_theme()
            self.filter_and_display_apps()  # Kartları yenile
        
        light_radio.toggled.connect(on_theme_changed)
        dark_radio.toggled.connect(on_theme_changed)
        
        dialog.exec()
    
    def resizeEvent(self, event):
        """Pencere boyutu değişti"""
        super().resizeEvent(event)
        if self.current_view_mode == ViewMode.GRID:
            self.update_grid_columns()
            self.display_apps()

def main():
    """Ana fonksiyon"""
    app = QApplication(sys.argv)
    app.setApplicationName("Cloud AppStore")
    app.setApplicationVersion("2.0.0")
    
    # Ana pencere
    window = CloudAppStore()
    window.show()
    
    return app.exec()

if __name__ == "__main__":
    sys.exit(main()) 


"""
Cloud Settings - Görünüm Sayfası
Tema, duvar kağıdı, dock ve UI ayarları
"""

from pathlib import Path
from PyQt6.QtWidgets import *
from PyQt6.QtCore import *
from .base_page import BasePage

class AppearancePage(BasePage):
    """Görünüm ayarları sayfası"""
    
    def __init__(self, kernel=None, preview_manager=None):
        super().__init__("appearance", "Görünüm", "🎨", kernel, preview_manager)
    
    def setup_ui(self):
        """UI kurulumu"""
        super().setup_ui()
        
        # Tema ayarları
        self.setup_theme_section()
        
        # Duvar kağıdı ayarları
        self.setup_wallpaper_section()
        
        # Dock ayarları
        self.setup_dock_section()
        
        # Topbar ayarları
        self.setup_topbar_section()
        
        # Renk ve efekt ayarları
        self.setup_colors_section()
        
        self.add_spacer()
    
    def setup_theme_section(self):
        """Tema ayarları bölümü"""
        group = self.add_group("Tema")
        layout = QVBoxLayout(group)
        
        # Tema seçimi
        theme_widget = QWidget()
        theme_layout = QHBoxLayout(theme_widget)
        theme_layout.setContentsMargins(0, 0, 0, 0)
        
        # Tema butonları
        self.theme_buttons = QButtonGroup()
        
        themes = [
            ("light", "☀️ Açık", "Açık renkli arayüz"),
            ("dark", "🌙 Koyu", "Koyu renkli arayüz"),
            ("auto", "🔄 Otomatik", "Sistem temasını takip et")
        ]
        
        for theme_id, theme_name, theme_desc in themes:
            theme_btn = QRadioButton()
            theme_btn.setObjectName(f"theme_{theme_id}")
            
            # Özel tema kartı
            card = QWidget()
            card.setFixedSize(120, 80)
            card_layout = QVBoxLayout(card)
            card_layout.setContentsMargins(8, 8, 8, 8)
            
            icon_label = QLabel(theme_name.split()[0])
            icon_label.setStyleSheet("font-size: 24px;")
            icon_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
            card_layout.addWidget(icon_label)
            
            name_label = QLabel(theme_name.split()[1])
            name_label.setStyleSheet("font-size: 12px; font-weight: 600;")
            name_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
            card_layout.addWidget(name_label)
            
            # Kart stili
            card.setStyleSheet(f"""
                QWidget {{
                    background-color: #ffffff;
                    border: 2px solid #e9ecef;
                    border-radius: 8px;
                }}
                
                QWidget:hover {{
                    border-color: #007bff;
                }}
            """)
            
            # Buton ve kart bağlantısı
            theme_btn.toggled.connect(lambda checked, tid=theme_id: self.on_theme_changed(tid, checked))
            card.mousePressEvent = lambda e, btn=theme_btn: btn.setChecked(True)
            
            self.theme_buttons.addButton(theme_btn)
            self.widgets[f"theme_{theme_id}"] = theme_btn
            
            btn_layout = QVBoxLayout()
            btn_layout.addWidget(theme_btn)
            btn_layout.addWidget(card)
            
            theme_layout.addLayout(btn_layout)
        
        theme_layout.addStretch()
        layout.addWidget(theme_widget)
        
        # Tema açıklaması
        self.theme_description = QLabel("Sistem temasını otomatik olarak takip eder")
        self.theme_description.setStyleSheet("""
            QLabel {
                font-size: 12px;
                color: #6c757d;
                margin-top: 10px;
            }
        """)
        layout.addWidget(self.theme_description)
    
    def setup_wallpaper_section(self):
        """Duvar kağıdı ayarları bölümü"""
        group = self.add_group("Duvar Kağıdı")
        layout = QVBoxLayout(group)
        
        # Duvar kağıdı seçimi
        self.add_file_picker(
            layout, 
            "Duvar Kağıdı", 
            "wallpaper_path",
            "Resim Dosyaları (*.png *.jpg *.jpeg *.bmp *.gif *.webp)",
            "Masaüstü arka plan resmi seçin"
        )
        
        # Duvar kağıdı modu
        self.add_combobox(
            layout,
            "Görüntüleme Modu",
            "wallpaper_mode",
            ["Sığdır", "Uzat", "Döşe", "Merkez", "Doldur"],
            "Duvar kağıdının nasıl görüntüleneceğini belirler"
        )
        
        # Duvar kağıdı önizlemesi
        self.wallpaper_preview = QLabel()
        self.wallpaper_preview.setFixedSize(200, 120)
        self.wallpaper_preview.setStyleSheet("""
            QLabel {
                background-color: #f8f9fa;
                border: 2px dashed #dee2e6;
                border-radius: 8px;
                color: #6c757d;
            }
        """)
        self.wallpaper_preview.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.wallpaper_preview.setText("Önizleme")
        
        preview_layout = QHBoxLayout()
        preview_layout.addWidget(QLabel("Önizleme:"))
        preview_layout.addWidget(self.wallpaper_preview)
        preview_layout.addStretch()
        
        layout.addLayout(preview_layout)
    
    def setup_dock_section(self):
        """Dock ayarları bölümü"""
        group = self.add_group("Dock")
        layout = QVBoxLayout(group)
        
        # Dock konumu
        self.add_combobox(
            layout,
            "Konum",
            "dock_position", 
            ["Alt", "Sol", "Sağ", "Üst"],
            "Dock'un ekrandaki konumu"
        )
        
        # Dock boyutu
        self.add_slider(
            layout,
            "Boyut",
            "dock_size",
            32, 128, 64,
            "Dock simgelerinin boyutu (piksel)"
        )
        
        # Otomatik gizleme
        self.add_checkbox(
            layout,
            "Otomatik gizle",
            "dock_autohide",
            "Dock'u kullanılmadığında otomatik olarak gizle"
        )
        
        # Dock efektleri
        self.add_checkbox(
            layout,
            "Büyütme efekti",
            "dock_magnification",
            "Fare üzerine gelindiğinde simgeleri büyüt"
        )
    
    def setup_topbar_section(self):
        """Topbar ayarları bölümü"""
        group = self.add_group("Üst Çubuk")
        layout = QVBoxLayout(group)
        
        # Topbar öğeleri
        self.add_checkbox(
            layout,
            "Saati göster",
            "show_clock",
            "Üst çubukta saat ve tarihi göster"
        )
        
        self.add_checkbox(
            layout,
            "Kullanıcı menüsünü göster", 
            "show_user",
            "Üst çubukta kullanıcı menüsünü göster"
        )
        
        self.add_checkbox(
            layout,
            "Bildirimleri göster",
            "show_notifications", 
            "Üst çubukta bildirim simgesini göster"
        )
        
        self.add_checkbox(
            layout,
            "Widget'ları göster",
            "show_widgets",
            "Üst çubukta widget butonlarını göster"
        )
    
    def setup_colors_section(self):
        """Renk ve efekt ayarları bölümü"""
        group = self.add_group("Renkler ve Efektler")
        layout = QVBoxLayout(group)
        
        # Vurgu rengi
        self.add_color_picker(
            layout,
            "Vurgu Rengi",
            "accent_color",
            "#007bff",
            "Butonlar ve vurgular için kullanılan ana renk"
        )
        
        # Şeffaflık
        self.add_checkbox(
            layout,
            "Şeffaflık efektleri",
            "transparency",
            "Pencere ve menülerde şeffaflık efektleri kullan"
        )
        
        # Animasyonlar
        self.add_checkbox(
            layout,
            "Animasyonları etkinleştir",
            "animations",
            "Pencere geçişleri ve UI animasyonları"
        )
        
        # Gölge efektleri
        self.add_checkbox(
            layout,
            "Gölge efektleri",
            "shadows",
            "Pencere ve menülerde gölge efektleri"
        )
    
    def on_theme_changed(self, theme_id: str, checked: bool):
        """Tema değişti"""
        if checked:
            self.on_setting_changed("theme", theme_id)
            
            # Tema açıklamasını güncelle
            descriptions = {
                "light": "Açık renkli arayüz kullanılıyor",
                "dark": "Koyu renkli arayüz kullanılıyor", 
                "auto": "Sistem temasını otomatik olarak takip eder"
            }
            self.theme_description.setText(descriptions.get(theme_id, ""))
    
    def update_ui(self):
        """UI'yi ayarlara göre güncelle"""
        super().update_ui()
        
        # Tema butonlarını güncelle
        theme = self.settings.get("theme", "auto")
        for theme_id in ["light", "dark", "auto"]:
            widget_key = f"theme_{theme_id}"
            if widget_key in self.widgets:
                self.widgets[widget_key].setChecked(theme_id == theme)
        
        # Duvar kağıdı önizlemesini güncelle
        self.update_wallpaper_preview()
    
    def update_wallpaper_preview(self):
        """Duvar kağıdı önizlemesini güncelle"""
        wallpaper_path = self.settings.get("wallpaper_path", "")
        
        if wallpaper_path and Path(wallpaper_path).exists():
            try:
                from PyQt6.QtGui import QPixmap
                
                pixmap = QPixmap(wallpaper_path)
                if not pixmap.isNull():
                    # Önizleme boyutuna ölçekle
                    scaled_pixmap = pixmap.scaled(
                        self.wallpaper_preview.size(),
                        Qt.AspectRatioMode.KeepAspectRatio,
                        Qt.TransformationMode.SmoothTransformation
                    )
                    self.wallpaper_preview.setPixmap(scaled_pixmap)
                else:
                    self.wallpaper_preview.setText("Geçersiz resim")
            except Exception as e:
                self.wallpaper_preview.setText("Yüklenemedi")
                self.logger.warning(f"Failed to load wallpaper preview: {e}")
        else:
            from PyQt6.QtGui import QPixmap
            self.wallpaper_preview.setText("Önizleme")
            self.wallpaper_preview.setPixmap(QPixmap())  # Temizle 


"""
Cloud Terminal Core Modülleri
Modern PyCloud OS Terminal Uygulaması
"""

from .terminal_app import CloudTerminal
from .terminal_ui import TerminalWidget, TabWidget
from .command_runner import CommandRunner
from .history import CommandHistory
from .autocomplete import AutoCompleter
from .themes import TerminalThemes

__all__ = [
    'CloudTerminal',
    'TerminalWidget',
    'TabWidget', 
    'CommandRunner',
    'CommandHistory',
    'AutoCompleter',
    'TerminalThemes'
] 


"""Option handling and Option management logic."""
from __future__ import annotations

import argparse
import enum
import functools
import logging
from collections.abc import Sequence
from typing import Any
from typing import Callable

from flake8 import utils
from flake8.plugins.finder import Plugins

LOG = logging.getLogger(__name__)

# represent a singleton of "not passed arguments".
# an enum is chosen to trick mypy
_ARG = enum.Enum("_ARG", "NO")


def _flake8_normalize(
    value: str,
    *args: str,
    comma_separated_list: bool = False,
    normalize_paths: bool = False,
) -> str | list[str]:
    ret: str | list[str] = value
    if comma_separated_list and isinstance(ret, str):
        ret = utils.parse_comma_separated_list(value)

    if normalize_paths:
        if isinstance(ret, str):
            ret = utils.normalize_path(ret, *args)
        else:
            ret = utils.normalize_paths(ret, *args)

    return ret


class Option:
    """Our wrapper around an argparse argument parsers to add features."""

    def __init__(
        self,
        short_option_name: str | _ARG = _ARG.NO,
        long_option_name: str | _ARG = _ARG.NO,
        # Options below are taken from argparse.ArgumentParser.add_argument
        action: str | type[argparse.Action] | _ARG = _ARG.NO,
        default: Any | _ARG = _ARG.NO,
        type: Callable[..., Any] | _ARG = _ARG.NO,
        dest: str | _ARG = _ARG.NO,
        nargs: int | str | _ARG = _ARG.NO,
        const: Any | _ARG = _ARG.NO,
        choices: Sequence[Any] | _ARG = _ARG.NO,
        help: str | _ARG = _ARG.NO,
        metavar: str | _ARG = _ARG.NO,
        required: bool | _ARG = _ARG.NO,
        # Options below here are specific to Flake8
        parse_from_config: bool = False,
        comma_separated_list: bool = False,
        normalize_paths: bool = False,
    ) -> None:
        """Initialize an Option instance.

        The following are all passed directly through to argparse.

        :param short_option_name:
            The short name of the option (e.g., ``-x``). This will be the
            first argument passed to ``ArgumentParser.add_argument``
        :param long_option_name:
            The long name of the option (e.g., ``--xtra-long-option``). This
            will be the second argument passed to
            ``ArgumentParser.add_argument``
        :param default:
            Default value of the option.
        :param dest:
            Attribute name to store parsed option value as.
        :param nargs:
            Number of arguments to parse for this option.
        :param const:
            Constant value to store on a common destination. Usually used in
            conjunction with ``action="store_const"``.
        :param choices:
            Possible values for the option.
        :param help:
            Help text displayed in the usage information.
        :param metavar:
            Name to use instead of the long option name for help text.
        :param required:
            Whether this option is required or not.

        The following options may be passed directly through to :mod:`argparse`
        but may need some massaging.

        :param type:
            A callable to normalize the type (as is the case in
            :mod:`argparse`).
        :param action:
            Any action allowed by :mod:`argparse`.

        The following parameters are for Flake8's option handling alone.

        :param parse_from_config:
            Whether or not this option should be parsed out of config files.
        :param comma_separated_list:
            Whether the option is a comma separated list when parsing from a
            config file.
        :param normalize_paths:
            Whether the option is expecting a path or list of paths and should
            attempt to normalize the paths to absolute paths.
        """
        if (
            long_option_name is _ARG.NO
            and short_option_name is not _ARG.NO
            and short_option_name.startswith("--")
        ):
            short_option_name, long_option_name = _ARG.NO, short_option_name

        # flake8 special type normalization
        if comma_separated_list or normalize_paths:
            type = functools.partial(
                _flake8_normalize,
                comma_separated_list=comma_separated_list,
                normalize_paths=normalize_paths,
            )

        self.short_option_name = short_option_name
        self.long_option_name = long_option_name
        self.option_args = [
            x
            for x in (short_option_name, long_option_name)
            if x is not _ARG.NO
        ]
        self.action = action
        self.default = default
        self.type = type
        self.dest = dest
        self.nargs = nargs
        self.const = const
        self.choices = choices
        self.help = help
        self.metavar = metavar
        self.required = required
        self.option_kwargs: dict[str, Any | _ARG] = {
            "action": self.action,
            "default": self.default,
            "type": self.type,
            "dest": self.dest,
            "nargs": self.nargs,
            "const": self.const,
            "choices": self.choices,
            "help": self.help,
            "metavar": self.metavar,
            "required": self.required,
        }

        # Set our custom attributes
        self.parse_from_config = parse_from_config
        self.comma_separated_list = comma_separated_list
        self.normalize_paths = normalize_paths

        self.config_name: str | None = None
        if parse_from_config:
            if long_option_name is _ARG.NO:
                raise ValueError(
                    "When specifying parse_from_config=True, "
                    "a long_option_name must also be specified."
                )
            self.config_name = long_option_name[2:].replace("-", "_")

        self._opt = None

    @property
    def filtered_option_kwargs(self) -> dict[str, Any]:
        """Return any actually-specified arguments."""
        return {
            k: v for k, v in self.option_kwargs.items() if v is not _ARG.NO
        }

    def __repr__(self) -> str:  # noqa: D105
        parts = []
        for arg in self.option_args:
            parts.append(arg)
        for k, v in self.filtered_option_kwargs.items():
            parts.append(f"{k}={v!r}")
        return f"Option({', '.join(parts)})"

    def normalize(self, value: Any, *normalize_args: str) -> Any:
        """Normalize the value based on the option configuration."""
        if self.comma_separated_list and isinstance(value, str):
            value = utils.parse_comma_separated_list(value)

        if self.normalize_paths:
            if isinstance(value, list):
                value = utils.normalize_paths(value, *normalize_args)
            else:
                value = utils.normalize_path(value, *normalize_args)

        return value

    def to_argparse(self) -> tuple[list[str], dict[str, Any]]:
        """Convert a Flake8 Option to argparse ``add_argument`` arguments."""
        return self.option_args, self.filtered_option_kwargs


class OptionManager:
    """Manage Options and OptionParser while adding post-processing."""

    def __init__(
        self,
        *,
        version: str,
        plugin_versions: str,
        parents: list[argparse.ArgumentParser],
        formatter_names: list[str],
    ) -> None:
        """Initialize an instance of an OptionManager."""
        self.formatter_names = formatter_names
        self.parser = argparse.ArgumentParser(
            prog="flake8",
            usage="%(prog)s [options] file file ...",
            parents=parents,
            epilog=f"Installed plugins: {plugin_versions}",
        )
        self.parser.add_argument(
            "--version",
            action="version",
            version=(
                f"{version} ({plugin_versions}) "
                f"{utils.get_python_version()}"
            ),
        )
        self.parser.add_argument("filenames", nargs="*", metavar="filename")

        self.config_options_dict: dict[str, Option] = {}
        self.options: list[Option] = []
        self.extended_default_ignore: list[str] = []
        self.extended_default_select: list[str] = []

        self._current_group: argparse._ArgumentGroup | None = None

    # TODO: maybe make this a free function to reduce api surface area
    def register_plugins(self, plugins: Plugins) -> None:
        """Register the plugin options (if needed)."""
        groups: dict[str, argparse._ArgumentGroup] = {}

        def _set_group(name: str) -> None:
            try:
                self._current_group = groups[name]
            except KeyError:
                group = self.parser.add_argument_group(name)
                self._current_group = groups[name] = group

        for loaded in plugins.all_plugins():
            add_options = getattr(loaded.obj, "add_options", None)
            if add_options:
                _set_group(loaded.plugin.package)
                add_options(self)

            if loaded.plugin.entry_point.group == "flake8.extension":
                self.extend_default_select([loaded.entry_name])

        # isn't strictly necessary, but seems cleaner
        self._current_group = None

    def add_option(self, *args: Any, **kwargs: Any) -> None:
        """Create and register a new option.

        See parameters for :class:`~flake8.options.manager.Option` for
        acceptable arguments to this method.

        .. note::

            ``short_option_name`` and ``long_option_name`` may be specified
            positionally as they are with argparse normally.
        """
        option = Option(*args, **kwargs)
        option_args, option_kwargs = option.to_argparse()
        if self._current_group is not None:
            self._current_group.add_argument(*option_args, **option_kwargs)
        else:
            self.parser.add_argument(*option_args, **option_kwargs)
        self.options.append(option)
        if option.parse_from_config:
            name = option.config_name
            assert name is not None
            self.config_options_dict[name] = option
            self.config_options_dict[name.replace("_", "-")] = option
        LOG.debug('Registered option "%s".', option)

    def extend_default_ignore(self, error_codes: Sequence[str]) -> None:
        """Extend the default ignore list with the error codes provided.

        :param error_codes:
            List of strings that are the error/warning codes with which to
            extend the default ignore list.
        """
        LOG.debug("Extending default ignore list with %r", error_codes)
        self.extended_default_ignore.extend(error_codes)

    def extend_default_select(self, error_codes: Sequence[str]) -> None:
        """Extend the default select list with the error codes provided.

        :param error_codes:
            List of strings that are the error/warning codes with which
            to extend the default select list.
        """
        LOG.debug("Extending default select list with %r", error_codes)
        self.extended_default_select.extend(error_codes)

    def parse_args(
        self,
        args: Sequence[str] | None = None,
        values: argparse.Namespace | None = None,
    ) -> argparse.Namespace:
        """Proxy to calling the OptionParser's parse_args method."""
        if values:
            self.parser.set_defaults(**vars(values))
        return self.parser.parse_args(args)



"""
Cloud Terminal - Modern UI Widget'ları
Terminal arayüzü, sekmeli yapı ve etkileşimli bileşenler
"""

import os
import sys
from typing import Optional, List, Dict, Any
from datetime import datetime

try:
    from PyQt6.QtWidgets import *
    from PyQt6.QtCore import *
    from PyQt6.QtGui import *
except ImportError:
    print("PyQt6 not available for terminal UI")
    sys.exit(1)

class TabWidget(QTabWidget):
    """Modern sekmeli terminal widget"""
    
    def __init__(self):
        super().__init__()
        self.setTabPosition(QTabWidget.TabPosition.North)
        self.setMovable(True)
        self.setTabsClosable(True)
        
        # Modern stil
        self.setStyleSheet("""
            QTabWidget::pane {
                border: 1px solid #3c3c3c;
                background-color: #1e1e1e;
                margin-top: -1px;
            }
            
            QTabBar::tab {
                background-color: #2d2d2d;
                color: #ffffff;
                border: 1px solid #3c3c3c;
                padding: 8px 16px;
                margin-right: 2px;
                border-top-left-radius: 4px;
                border-top-right-radius: 4px;
                min-width: 100px;
            }
            
            QTabBar::tab:selected {
                background-color: #1e1e1e;
                border-bottom-color: #1e1e1e;
                color: #00ff88;
            }
            
            QTabBar::tab:hover {
                background-color: #3c3c3c;
            }
            
            QTabBar::close-button {
                image: none;
                background-color: #ff6b6b;
                border-radius: 6px;
                width: 12px;
                height: 12px;
            }
            
            QTabBar::close-button:hover {
                background-color: #ff5252;
            }
        """)

class TerminalOutput(QTextEdit):
    """Terminal çıktı widget'ı"""
    
    def __init__(self):
        super().__init__()
        self.setReadOnly(True)
        self.setFont(QFont("Consolas", 13))
        self.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAsNeeded)
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAsNeeded)
        
        # Terminal renkleri
        self.default_color = "#ffffff"
        self.error_color = "#ff6b6b"
        self.success_color = "#51cf66"
        self.warning_color = "#ffd43b"
        self.info_color = "#74c0fc"
        
        # Modern terminal stili
        self.setStyleSheet("""
            QTextEdit {
                background-color: #1e1e1e;
                color: #ffffff;
                border: none;
                font-family: 'Consolas', 'Monaco', 'Courier New', monospace;
                font-size: 13px;
                line-height: 1.4;
                padding: 10px;
            }
            
            QScrollBar:vertical {
                background-color: #2d2d2d;
                width: 12px;
                border-radius: 6px;
            }
            
            QScrollBar::handle:vertical {
                background-color: #5c5c5c;
                border-radius: 6px;
                min-height: 20px;
            }
            
            QScrollBar::handle:vertical:hover {
                background-color: #7c7c7c;
            }
        """)
    
    def append_text(self, text: str, color: str = None):
        """Renkli metin ekle"""
        if not color:
            color = self.default_color
        
        cursor = self.textCursor()
        cursor.movePosition(QTextCursor.MoveOperation.End)
        
        # Renk formatı
        format = QTextCharFormat()
        format.setForeground(QColor(color))
        
        cursor.setCharFormat(format)
        cursor.insertText(text)
        
        # Scroll to bottom
        self.setTextCursor(cursor)
        self.ensureCursorVisible()
    
    def append_output(self, text: str):
        """Normal çıktı ekle"""
        self.append_text(text, self.default_color)
    
    def append_error(self, text: str):
        """Hata çıktısı ekle"""
        self.append_text(text, self.error_color)
    
    def append_success(self, text: str):
        """Başarı mesajı ekle"""
        self.append_text(text, self.success_color)
    
    def append_warning(self, text: str):
        """Uyarı mesajı ekle"""
        self.append_text(text, self.warning_color)
    
    def append_info(self, text: str):
        """Bilgi mesajı ekle"""
        self.append_text(text, self.info_color)
    
    def clear_output(self):
        """Çıktıyı temizle"""
        self.clear()
    
    def get_all_text(self) -> str:
        """Tüm metni al"""
        return self.toPlainText()

class TerminalInput(QLineEdit):
    """Terminal giriş widget'ı"""
    
    # Sinyaller
    command_entered = pyqtSignal(str)
    history_up = pyqtSignal()
    history_down = pyqtSignal()
    tab_pressed = pyqtSignal(str)  # Autocomplete için
    
    def __init__(self):
        super().__init__()
        self.setFont(QFont("Consolas", 13))
        self.setPlaceholderText("Enter command...")
        
        # Modern stil
        self.setStyleSheet("""
            QLineEdit {
                background-color: #2d2d2d;
                color: #ffffff;
                border: 2px solid #3c3c3c;
                border-radius: 6px;
                padding: 8px 12px;
                font-family: 'Consolas', 'Monaco', 'Courier New', monospace;
                font-size: 13px;
            }
            
            QLineEdit:focus {
                border-color: #00ff88;
                background-color: #1e1e1e;
            }
            
            QLineEdit::placeholder {
                color: #6c757d;
            }
        """)
        
        # Enter tuşu için bağlantı
        self.returnPressed.connect(self.on_enter_pressed)
    
    def keyPressEvent(self, event):
        """Klavye olayları"""
        key = event.key()
        
        if key == Qt.Key.Key_Return or key == Qt.Key.Key_Enter:
            self.on_enter_pressed()
        elif key == Qt.Key.Key_Up:
            self.history_up.emit()
        elif key == Qt.Key.Key_Down:
            self.history_down.emit()
        elif key == Qt.Key.Key_Tab:
            self.tab_pressed.emit(self.text())
            event.accept()
            return
        else:
            super().keyPressEvent(event)
    
    def on_enter_pressed(self):
        """Enter tuşuna basıldı"""
        command = self.text().strip()
        if command:
            self.command_entered.emit(command)
            self.clear()

class AutoCompletePopup(QListWidget):
    """Autocomplete popup widget'ı"""
    
    # Sinyaller
    completion_selected = pyqtSignal(str)
    
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setWindowFlags(Qt.WindowType.Popup)
        self.setMaximumHeight(200)
        self.setMinimumWidth(300)
        
        # Modern stil
        self.setStyleSheet("""
            QListWidget {
                background-color: #2d2d2d;
                color: #ffffff;
                border: 1px solid #3c3c3c;
                border-radius: 6px;
                padding: 4px;
            }
            
            QListWidget::item {
                padding: 6px 10px;
                border-radius: 4px;
            }
            
            QListWidget::item:selected {
                background-color: #007acc;
                color: #ffffff;
            }
            
            QListWidget::item:hover {
                background-color: #3c3c3c;
            }
        """)
        
        # Bağlantılar
        self.itemClicked.connect(self.on_item_clicked)
    
    def show_completions(self, completions: List[str], position: QPoint):
        """Tamamlama seçeneklerini göster"""
        self.clear()
        
        if not completions:
            self.hide()
            return
        
        for completion in completions:
            self.addItem(completion)
        
        # Pozisyonu ayarla
        self.move(position)
        self.show()
        
        # İlk öğeyi seç
        if self.count() > 0:
            self.setCurrentRow(0)
    
    def on_item_clicked(self, item):
        """Öğe tıklandı"""
        self.completion_selected.emit(item.text())
        self.hide()
    
    def keyPressEvent(self, event):
        """Klavye olayları"""
        key = event.key()
        
        if key == Qt.Key.Key_Return or key == Qt.Key.Key_Enter:
            current_item = self.currentItem()
            if current_item:
                self.completion_selected.emit(current_item.text())
                self.hide()
        elif key == Qt.Key.Key_Escape:
            self.hide()
        else:
            super().keyPressEvent(event)

class TerminalWidget(QWidget):
    """Ana terminal widget'ı"""
    
    def __init__(self, kernel=None, command_runner=None, history_manager=None, autocompleter=None):
        super().__init__()
        self.kernel = kernel
        self.command_runner = command_runner
        self.history_manager = history_manager
        self.autocompleter = autocompleter
        
        # Terminal durumu
        self.current_directory = os.getcwd()
        self.session_id = f"terminal_{datetime.now().strftime('%Y%m%d_%H%M%S')}"
        
        # UI kurulumu
        self.setup_ui()
        self.setup_connections()
        
        # Hoş geldin mesajı
        self.show_welcome_message()
    
    def setup_ui(self):
        """UI kurulumu"""
        layout = QVBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(0)
        
        # Terminal çıktısı
        self.output = TerminalOutput()
        layout.addWidget(self.output, 1)
        
        # Alt panel
        input_panel = QWidget()
        input_panel.setFixedHeight(50)
        input_layout = QHBoxLayout(input_panel)
        input_layout.setContentsMargins(10, 8, 10, 8)
        
        # Prompt label
        self.prompt_label = QLabel("$")
        self.prompt_label.setFont(QFont("Consolas", 13, QFont.Weight.Bold))
        self.prompt_label.setStyleSheet("color: #00ff88; margin-right: 5px;")
        input_layout.addWidget(self.prompt_label)
        
        # Komut girişi
        self.input = TerminalInput()
        input_layout.addWidget(self.input, 1)
        
        layout.addWidget(input_panel)
        
        # Autocomplete popup
        self.autocomplete_popup = AutoCompletePopup(self)
        
        # Modern panel stili
        input_panel.setStyleSheet("""
            QWidget {
                background-color: #2d2d2d;
                border-top: 1px solid #3c3c3c;
            }
        """)
    
    def setup_connections(self):
        """Sinyal bağlantıları"""
        self.input.command_entered.connect(self.execute_command)
        self.input.history_up.connect(self.history_up)
        self.input.history_down.connect(self.history_down)
        self.input.tab_pressed.connect(self.show_autocomplete)
        self.autocomplete_popup.completion_selected.connect(self.apply_completion)
    
    def show_welcome_message(self):
        """Hoş geldin mesajı"""
        welcome_text = f"""
╭─────────────────────────────────────────────────────────────╮
│                    🌩️  Cloud Terminal v2.0.0                │
│                     PyCloud OS Terminal                     │
╰─────────────────────────────────────────────────────────────╯

Welcome to Cloud Terminal! Type 'help' for available commands.
Current directory: {self.current_directory}
Session: {self.session_id}

"""
        self.output.append_info(welcome_text)
        self.show_prompt()
    
    def show_prompt(self):
        """Prompt göster"""
        # Mevcut dizin adını al
        dir_name = os.path.basename(self.current_directory) or "/"
        prompt_text = f"\n{dir_name} $ "
        self.output.append_text(prompt_text, "#00ff88")
    
    def execute_command(self, command: str):
        """Komut çalıştır"""
        # Komutu göster
        self.output.append_text(command + "\n", "#ffffff")
        
        # Geçmişe ekle
        if self.history_manager:
            self.history_manager.add_command(command)
        
        # Komut çalıştır
        if self.command_runner:
            try:
                result = self.command_runner.execute_command(command, self.current_directory)
                
                # Sonucu göster
                if result.get('output'):
                    self.output.append_output(result['output'] + "\n")
                
                if result.get('error'):
                    self.output.append_error(result['error'] + "\n")
                
                # Dizin değişikliği kontrolü
                if result.get('new_directory'):
                    self.current_directory = result['new_directory']
                
                # Özel komutlar
                if result.get('clear'):
                    self.clear_output()
                    return
                
                if result.get('exit'):
                    self.close_terminal()
                    return
                
            except Exception as e:
                self.output.append_error(f"Command execution error: {e}\n")
        
        # Yeni prompt göster
        self.show_prompt()
    
    def history_up(self):
        """Geçmişte yukarı git"""
        if self.history_manager:
            command = self.history_manager.get_previous()
            if command:
                self.input.setText(command)
    
    def history_down(self):
        """Geçmişte aşağı git"""
        if self.history_manager:
            command = self.history_manager.get_next()
            if command:
                self.input.setText(command)
            else:
                self.input.clear()
    
    def show_autocomplete(self, partial_command: str):
        """Autocomplete göster"""
        if not self.autocompleter or not partial_command.strip():
            return
        
        completions = self.autocompleter.get_completions(partial_command, self.current_directory)
        
        if completions:
            # Popup pozisyonunu hesapla
            input_pos = self.input.mapToGlobal(self.input.rect().bottomLeft())
            self.autocomplete_popup.show_completions(completions, input_pos)
    
    def apply_completion(self, completion: str):
        """Tamamlamayı uygula"""
        current_text = self.input.text()
        words = current_text.split()
        
        if words:
            # Son kelimeyi değiştir
            words[-1] = completion
            self.input.setText(" ".join(words))
        else:
            self.input.setText(completion)
        
        # Cursor'u sona taşı
        self.input.setCursorPosition(len(self.input.text()))
    
    def clear_output(self):
        """Çıktıyı temizle"""
        self.output.clear_output()
        self.show_welcome_message()
    
    def get_output_text(self) -> str:
        """Çıktı metnini al"""
        return self.output.get_all_text()
    
    def get_current_directory(self) -> str:
        """Mevcut dizini al"""
        return self.current_directory
    
    def set_font_size(self, size: int):
        """Font boyutunu ayarla"""
        font = QFont("Consolas", size)
        self.output.setFont(font)
        self.input.setFont(font)
        self.prompt_label.setFont(QFont("Consolas", size, QFont.Weight.Bold))
    
    def copy_selection(self):
        """Seçili metni kopyala"""
        if self.output.textCursor().hasSelection():
            clipboard = QApplication.clipboard()
            clipboard.setText(self.output.textCursor().selectedText())
    
    def paste_text(self):
        """Metin yapıştır"""
        clipboard = QApplication.clipboard()
        text = clipboard.text()
        if text:
            self.input.insert(text)
    
    def close_terminal(self):
        """Terminal'i kapat"""
        # Parent window'a kapat sinyali gönder
        parent_window = self.window()
        if hasattr(parent_window, 'close_current_session'):
            parent_window.close_current_session()
    
    def cleanup(self):
        """Temizlik işlemleri"""
        # Autocomplete popup'ı kapat
        if self.autocomplete_popup:
            self.autocomplete_popup.hide()
        
        # Session'ı temizle
        if self.history_manager:
            self.history_manager.save_history() 


from __future__ import annotations

from virtualenv.activation.via_template import ViaTemplateActivator


class FishActivator(ViaTemplateActivator):
    def templates(self):
        yield "activate.fish"


__all__ = [
    "FishActivator",
]



from __future__ import annotations


def is_module_from_legacy_bundled_package(module: str) -> bool:
    top_level = module.split(".", 1)[0]
    return top_level in legacy_bundled_packages


def stub_distribution_name(module: str) -> str | None:
    top_level = module.split(".", 1)[0]

    dist = legacy_bundled_packages.get(top_level)
    if dist:
        return dist
    dist = non_bundled_packages_flat.get(top_level)
    if dist:
        return dist

    if top_level in non_bundled_packages_namespace:
        namespace = non_bundled_packages_namespace[top_level]
        components = module.split(".")
        for i in range(len(components), 0, -1):
            module = ".".join(components[:i])
            dist = namespace.get(module)
            if dist:
                return dist

    return None


# Stubs for these third-party packages used to be shipped with mypy.
#
# Map package name to PyPI stub distribution name.
legacy_bundled_packages: dict[str, str] = {
    "aiofiles": "types-aiofiles",
    "bleach": "types-bleach",
    "cachetools": "types-cachetools",
    "click_spinner": "types-click-spinner",
    "croniter": "types-croniter",
    "dateparser": "types-dateparser",
    "dateutil": "types-python-dateutil",
    "decorator": "types-decorator",
    "deprecated": "types-Deprecated",
    "docutils": "types-docutils",
    "first": "types-first",
    "markdown": "types-Markdown",
    "mock": "types-mock",
    "OpenSSL": "types-pyOpenSSL",
    "paramiko": "types-paramiko",
    "polib": "types-polib",
    "pycurl": "types-pycurl",
    "pymysql": "types-PyMySQL",
    "pyrfc3339": "types-pyRFC3339",
    "pytz": "types-pytz",
    "requests": "types-requests",
    "retry": "types-retry",
    "simplejson": "types-simplejson",
    "singledispatch": "types-singledispatch",
    "six": "types-six",
    "tabulate": "types-tabulate",
    "toml": "types-toml",
    "ujson": "types-ujson",
    "waitress": "types-waitress",
    "yaml": "types-PyYAML",
}

# Map package name to PyPI stub distribution name from typeshed.
# Stubs for these packages were never bundled with mypy. Don't
# include packages that have a release that includes PEP 561 type
# information.
#
# Note that these packages are omitted for now:
#   pika:       typeshed's stubs are on PyPI as types-pika-ts.
#               types-pika already exists on PyPI, and is more complete in many ways,
#               but is a non-typeshed stubs package.
non_bundled_packages_flat: dict[str, str] = {
    "_cffi_backend": "types-cffi",
    "_win32typing": "types-pywin32",
    "antlr4": "types-antlr4-python3-runtime",
    "assertpy": "types-assertpy",
    "atheris": "types-atheris",
    "authlib": "types-Authlib",
    "aws_xray_sdk": "types-aws-xray-sdk",
    "boltons": "types-boltons",
    "braintree": "types-braintree",
    "bs4": "types-beautifulsoup4",
    "bugbear": "types-flake8-bugbear",
    "caldav": "types-caldav",
    "capturer": "types-capturer",
    "cffi": "types-cffi",
    "chevron": "types-chevron",
    "click_default_group": "types-click-default-group",
    "click_log": "types-click-log",
    "click_web": "types-click-web",
    "colorama": "types-colorama",
    "commctrl": "types-pywin32",
    "commonmark": "types-commonmark",
    "consolemenu": "types-console-menu",
    "corus": "types-corus",  # codespell:ignore corus
    "cronlog": "types-python-crontab",
    "crontab": "types-python-crontab",
    "crontabs": "types-python-crontab",
    "datemath": "types-python-datemath",
    "dateparser_data": "types-dateparser",
    "dde": "types-pywin32",
    "defusedxml": "types-defusedxml",
    "docker": "types-docker",
    "dockerfile_parse": "types-dockerfile-parse",
    "editdistance": "types-editdistance",
    "entrypoints": "types-entrypoints",
    "exifread": "types-ExifRead",
    "fanstatic": "types-fanstatic",
    "farmhash": "types-pyfarmhash",
    "flake8_builtins": "types-flake8-builtins",
    "flake8_docstrings": "types-flake8-docstrings",
    "flake8_rst_docstrings": "types-flake8-rst-docstrings",
    "flake8_simplify": "types-flake8-simplify",
    "flake8_typing_imports": "types-flake8-typing-imports",
    "flake8": "types-flake8",
    "flask_cors": "types-Flask-Cors",
    "flask_migrate": "types-Flask-Migrate",
    "flask_socketio": "types-Flask-SocketIO",
    "fpdf": "types-fpdf2",
    "gdb": "types-gdb",
    "gevent": "types-gevent",
    "greenlet": "types-greenlet",
    "hdbcli": "types-hdbcli",
    "html5lib": "types-html5lib",
    "httplib2": "types-httplib2",
    "humanfriendly": "types-humanfriendly",
    "hvac": "types-hvac",
    "ibm_db": "types-ibm-db",
    "icalendar": "types-icalendar",
    "import_export": "types-django-import-export",
    "influxdb_client": "types-influxdb-client",
    "inifile": "types-inifile",
    "isapi": "types-pywin32",
    "jack": "types-JACK-Client",
    "jenkins": "types-python-jenkins",
    "Jetson": "types-Jetson.GPIO",
    "jks": "types-pyjks",
    "jmespath": "types-jmespath",
    "jose": "types-python-jose",
    "jsonschema": "types-jsonschema",
    "jwcrypto": "types-jwcrypto",
    "keyboard": "types-keyboard",
    "ldap3": "types-ldap3",
    "lupa": "types-lupa",
    "lzstring": "types-lzstring",
    "m3u8": "types-m3u8",
    "mmapfile": "types-pywin32",
    "mmsystem": "types-pywin32",
    "mypy_extensions": "types-mypy-extensions",
    "MySQLdb": "types-mysqlclient",
    "nanoid": "types-nanoid",
    "nanoleafapi": "types-nanoleafapi",
    "netaddr": "types-netaddr",
    "netifaces": "types-netifaces",
    "networkx": "types-networkx",
    "nmap": "types-python-nmap",
    "ntsecuritycon": "types-pywin32",
    "oauthlib": "types-oauthlib",
    "objgraph": "types-objgraph",
    "odbc": "types-pywin32",
    "olefile": "types-olefile",
    "openpyxl": "types-openpyxl",
    "opentracing": "types-opentracing",
    "parsimonious": "types-parsimonious",
    "passlib": "types-passlib",
    "passpy": "types-passpy",
    "peewee": "types-peewee",
    "pep8ext_naming": "types-pep8-naming",
    "perfmon": "types-pywin32",
    "pexpect": "types-pexpect",
    "playhouse": "types-peewee",
    "portpicker": "types-portpicker",
    "psutil": "types-psutil",
    "psycopg2": "types-psycopg2",
    "pyasn1": "types-pyasn1",
    "pyaudio": "types-pyaudio",
    "pyautogui": "types-PyAutoGUI",
    "pycocotools": "types-pycocotools",
    "pyflakes": "types-pyflakes",
    "pygit2": "types-pygit2",
    "pygments": "types-Pygments",
    "pyi_splash": "types-pyinstaller",
    "PyInstaller": "types-pyinstaller",
    "pynput": "types-pynput",
    "pyscreeze": "types-PyScreeze",
    "pysftp": "types-pysftp",
    "pytest_lazyfixture": "types-pytest-lazy-fixture",
    "python_http_client": "types-python-http-client",
    "pythoncom": "types-pywin32",
    "pythonwin": "types-pywin32",
    "pywintypes": "types-pywin32",
    "qrbill": "types-qrbill",
    "qrcode": "types-qrcode",
    "regex": "types-regex",
    "regutil": "types-pywin32",
    "reportlab": "types-reportlab",
    "requests_oauthlib": "types-requests-oauthlib",
    "RPi": "types-RPi.GPIO",
    "s2clientprotocol": "types-s2clientprotocol",
    "sass": "types-libsass",
    "sassutils": "types-libsass",
    "seaborn": "types-seaborn",
    "send2trash": "types-Send2Trash",
    "serial": "types-pyserial",
    "servicemanager": "types-pywin32",
    "setuptools": "types-setuptools",
    "shapely": "types-shapely",
    "slumber": "types-slumber",
    "sspicon": "types-pywin32",
    "str2bool": "types-str2bool",
    "tensorflow": "types-tensorflow",
    "tgcrypto": "types-TgCrypto",
    "timer": "types-pywin32",
    "toposort": "types-toposort",
    "tqdm": "types-tqdm",
    "translationstring": "types-translationstring",
    "tree_sitter_languages": "types-tree-sitter-languages",
    "ttkthemes": "types-ttkthemes",
    "unidiff": "types-unidiff",
    "untangle": "types-untangle",
    "usersettings": "types-usersettings",
    "uwsgi": "types-uWSGI",
    "uwsgidecorators": "types-uWSGI",
    "vobject": "types-vobject",
    "webob": "types-WebOb",
    "whatthepatch": "types-whatthepatch",
    "win2kras": "types-pywin32",
    "win32": "types-pywin32",
    "win32api": "types-pywin32",
    "win32clipboard": "types-pywin32",
    "win32com": "types-pywin32",
    "win32comext": "types-pywin32",
    "win32con": "types-pywin32",
    "win32console": "types-pywin32",
    "win32cred": "types-pywin32",
    "win32crypt": "types-pywin32",
    "win32cryptcon": "types-pywin32",
    "win32event": "types-pywin32",
    "win32evtlog": "types-pywin32",
    "win32evtlogutil": "types-pywin32",
    "win32file": "types-pywin32",
    "win32gui_struct": "types-pywin32",
    "win32gui": "types-pywin32",
    "win32help": "types-pywin32",
    "win32inet": "types-pywin32",
    "win32inetcon": "types-pywin32",
    "win32job": "types-pywin32",
    "win32lz": "types-pywin32",
    "win32net": "types-pywin32",
    "win32netcon": "types-pywin32",
    "win32pdh": "types-pywin32",
    "win32pdhquery": "types-pywin32",
    "win32pipe": "types-pywin32",
    "win32print": "types-pywin32",
    "win32process": "types-pywin32",
    "win32profile": "types-pywin32",
    "win32ras": "types-pywin32",
    "win32security": "types-pywin32",
    "win32service": "types-pywin32",
    "win32serviceutil": "types-pywin32",
    "win32timezone": "types-pywin32",
    "win32trace": "types-pywin32",
    "win32transaction": "types-pywin32",
    "win32ts": "types-pywin32",
    "win32ui": "types-pywin32",
    "win32uiole": "types-pywin32",
    "win32verstamp": "types-pywin32",
    "win32wnet": "types-pywin32",
    "winerror": "types-pywin32",
    "winioctlcon": "types-pywin32",
    "winnt": "types-pywin32",
    "winperf": "types-pywin32",
    "winxpgui": "types-pywin32",
    "winxptheme": "types-pywin32",
    "workalendar": "types-workalendar",
    "wtforms": "types-WTForms",
    "wurlitzer": "types-wurlitzer",
    "xdg": "types-pyxdg",
    "xdgenvpy": "types-xdgenvpy",
    "Xlib": "types-python-xlib",
    "xmltodict": "types-xmltodict",
    "zstd": "types-zstd",
    "zxcvbn": "types-zxcvbn",
    # Stub packages that are not from typeshed
    # Since these can be installed automatically via --install-types, we have a high trust bar
    # for additions here
    "pandas": "pandas-stubs",  # https://github.com/pandas-dev/pandas-stubs
    "lxml": "lxml-stubs",  # https://github.com/lxml/lxml-stubs
    "scipy": "scipy-stubs",  # https://github.com/scipy/scipy-stubs
}


non_bundled_packages_namespace: dict[str, dict[str, str]] = {
    "backports": {"backports.ssl_match_hostname": "types-backports.ssl_match_hostname"},
    "google": {"google.cloud.ndb": "types-google-cloud-ndb", "google.protobuf": "types-protobuf"},
    "paho": {"paho.mqtt": "types-paho-mqtt"},
}



"""
PyCloud OS Notification Manager
PyCloud OS sisteminde kullanıcıya toast-style anlık bildirim sunan sistem modülü
"""

import time
import logging
import threading
from pathlib import Path
from typing import Dict, List, Optional, Callable, Any
from datetime import datetime
from dataclasses import dataclass, asdict
from enum import Enum

try:
    from PyQt6.QtWidgets import (QWidget, QVBoxLayout, QHBoxLayout, QLabel, 
                                QPushButton, QFrame, QGraphicsOpacityEffect,
                                QApplication)
    from PyQt6.QtCore import Qt, QTimer, QPropertyAnimation, QEasingCurve, pyqtSignal
    from PyQt6.QtGui import QFont, QPixmap, QPainter, QColor, QBrush
    PYQT_AVAILABLE = True
except ImportError:
    PYQT_AVAILABLE = False

class NotificationType(Enum):
    """Bildirim türleri"""
    INFO = "info"
    SUCCESS = "success"
    WARNING = "warning"
    ERROR = "error"
    SYSTEM = "system"
    APP = "app"
    SECURITY = "security"
    UPDATE = "update"

class NotificationPriority(Enum):
    """Bildirim öncelikleri"""
    LOW = "low"
    NORMAL = "normal"
    HIGH = "high"
    URGENT = "urgent"

@dataclass
class NotificationData:
    """Bildirim veri sınıfı"""
    id: str
    title: str
    message: str
    notification_type: NotificationType
    priority: NotificationPriority
    timestamp: str
    source: str = "system"
    icon: str = "ℹ️"
    actions: List[Dict] = None
    persistent: bool = False
    sound: bool = True
    auto_dismiss: bool = True
    dismiss_delay: float = 5.0
    
    def __post_init__(self):
        if self.actions is None:
            self.actions = []
    
    def to_dict(self) -> Dict:
        """Dict'e çevir"""
        data = asdict(self)
        data['notification_type'] = self.notification_type.value
        data['priority'] = self.priority.value
        return data

class NotificationToast(QWidget):
    """Toast bildirim widget'ı"""
    
    # Sinyaller
    dismissed = pyqtSignal(str)  # notification_id
    action_clicked = pyqtSignal(str, str)  # notification_id, action_id
    
    def __init__(self, notification: NotificationData):
        super().__init__()
        self.notification = notification
        self.logger = logging.getLogger("NotificationToast")
        
        self.setWindowFlags(Qt.WindowType.FramelessWindowHint | 
                           Qt.WindowType.WindowStaysOnTopHint |
                           Qt.WindowType.Tool)
        self.setAttribute(Qt.WidgetAttribute.WA_TranslucentBackground)
        
        self.setup_ui()
        self.setup_animations()
        
        # Otomatik kapanma
        if notification.auto_dismiss and not notification.persistent:
            QTimer.singleShot(int(notification.dismiss_delay * 1000), self.dismiss)
    
    def setup_ui(self):
        """Arayüzü kur"""
        self.setFixedSize(350, 100)
        
        # Ana layout
        main_layout = QVBoxLayout(self)
        main_layout.setContentsMargins(0, 0, 0, 0)
        
        # Toast container
        self.toast_frame = QFrame()
        self.toast_frame.setStyleSheet(self._get_toast_style())
        
        # İçerik layout
        content_layout = QHBoxLayout(self.toast_frame)
        content_layout.setContentsMargins(15, 10, 15, 10)
        content_layout.setSpacing(10)
        
        # İkon
        icon_label = QLabel(self.notification.icon)
        icon_label.setFont(QFont("Arial", 20))
        icon_label.setFixedSize(30, 30)
        icon_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        content_layout.addWidget(icon_label)
        
        # Metin alanı
        text_layout = QVBoxLayout()
        text_layout.setSpacing(2)
        
        # Başlık
        title_label = QLabel(self.notification.title)
        title_label.setFont(QFont("Arial", 12, QFont.Weight.Bold))
        title_label.setStyleSheet("color: #ffffff;")
        text_layout.addWidget(title_label)
        
        # Mesaj
        message_label = QLabel(self.notification.message)
        message_label.setFont(QFont("Arial", 10))
        message_label.setStyleSheet("color: #cccccc;")
        message_label.setWordWrap(True)
        text_layout.addWidget(message_label)
        
        content_layout.addLayout(text_layout)
        
        # Kapat butonu
        if not self.notification.persistent:
            close_btn = QPushButton("✕")
            close_btn.setFixedSize(20, 20)
            close_btn.setStyleSheet("""
                QPushButton {
                    background-color: transparent;
                    border: none;
                    color: #ffffff;
                    font-size: 12px;
                }
                QPushButton:hover {
                    background-color: rgba(255, 255, 255, 0.2);
                    border-radius: 10px;
                }
            """)
            close_btn.clicked.connect(self.dismiss)
            content_layout.addWidget(close_btn)
        
        main_layout.addWidget(self.toast_frame)
        
        # Aksiyon butonları
        if self.notification.actions:
            self._add_action_buttons(main_layout)
    
    def _get_toast_style(self) -> str:
        """Toast stilini al"""
        # Türe göre renk
        colors = {
            NotificationType.INFO: "#2196F3",
            NotificationType.SUCCESS: "#4CAF50",
            NotificationType.WARNING: "#FF9800",
            NotificationType.ERROR: "#F44336",
            NotificationType.SYSTEM: "#9C27B0",
            NotificationType.APP: "#00BCD4",
            NotificationType.SECURITY: "#FF5722",
            NotificationType.UPDATE: "#3F51B5"
        }
        
        color = colors.get(self.notification.notification_type, "#2196F3")
        
        return f"""
            QFrame {{
                background-color: rgba(45, 45, 45, 0.95);
                border: 2px solid {color};
                border-radius: 12px;
            }}
        """
    
    def _add_action_buttons(self, layout):
        """Aksiyon butonlarını ekle"""
        if not self.notification.actions:
            return
        
        actions_frame = QFrame()
        actions_layout = QHBoxLayout(actions_frame)
        actions_layout.setContentsMargins(10, 5, 10, 10)
        
        for action in self.notification.actions:
            btn = QPushButton(action.get("title", "Action"))
            btn.setStyleSheet("""
                QPushButton {
                    background-color: rgba(255, 255, 255, 0.1);
                    border: 1px solid rgba(255, 255, 255, 0.3);
                    border-radius: 6px;
                    color: #ffffff;
                    padding: 5px 15px;
                    font-size: 10px;
                }
                QPushButton:hover {
                    background-color: rgba(255, 255, 255, 0.2);
                }
            """)
            
            action_id = action.get("id", "default")
            btn.clicked.connect(lambda checked, aid=action_id: self._on_action_clicked(aid))
            actions_layout.addWidget(btn)
        
        layout.addWidget(actions_frame)
    
    def _on_action_clicked(self, action_id: str):
        """Aksiyon tıklama"""
        self.action_clicked.emit(self.notification.id, action_id)
        self.dismiss()
    
    def setup_animations(self):
        """Animasyonları kur"""
        # Opacity efekti
        self.opacity_effect = QGraphicsOpacityEffect()
        self.setGraphicsEffect(self.opacity_effect)
        
        # Fade in animasyonu
        self.fade_in_animation = QPropertyAnimation(self.opacity_effect, b"opacity")
        self.fade_in_animation.setDuration(300)
        self.fade_in_animation.setStartValue(0.0)
        self.fade_in_animation.setEndValue(1.0)
        self.fade_in_animation.setEasingCurve(QEasingCurve.Type.OutCubic)
        
        # Fade out animasyonu
        self.fade_out_animation = QPropertyAnimation(self.opacity_effect, b"opacity")
        self.fade_out_animation.setDuration(200)
        self.fade_out_animation.setStartValue(1.0)
        self.fade_out_animation.setEndValue(0.0)
        self.fade_out_animation.setEasingCurve(QEasingCurve.Type.InCubic)
        self.fade_out_animation.finished.connect(self.close)
    
    def show_animated(self):
        """Animasyonlu göster"""
        self.show()
        self.fade_in_animation.start()
    
    def dismiss(self):
        """Bildirimi kapat"""
        self.dismissed.emit(self.notification.id)
        self.fade_out_animation.start()

class NotificationManager:
    """Bildirim yöneticisi"""
    
    def __init__(self, kernel=None):
        self.kernel = kernel
        self.logger = logging.getLogger("NotificationManager")
        
        # Bildirim depolama
        self.notifications: Dict[str, NotificationData] = {}
        self.notification_history: List[NotificationData] = []
        self.active_toasts: Dict[str, NotificationToast] = {}
        
        # Ayarlar
        self.max_visible_toasts = 3
        self.max_history_size = 50
        self.toast_position = "top-right"  # top-right, top-left, bottom-right, bottom-left
        self.sounds_enabled = True
        self.animations_enabled = True
        
        # Toast kuyruğu
        self.toast_queue: List[NotificationData] = []
        self.queue_timer = None
        
        # Callbacks
        self.callbacks: Dict[str, List[Callable]] = {
            "notification_shown": [],
            "notification_dismissed": [],
            "action_clicked": []
        }
        
        # PyQt6 varsa timer kur
        if PYQT_AVAILABLE:
            self.queue_timer = QTimer()
            self.queue_timer.timeout.connect(self._process_queue)
            self.queue_timer.start(500)  # 500ms
    
    def show_notification(self, title: str, message: str, 
                         notification_type: NotificationType = NotificationType.INFO,
                         priority: NotificationPriority = NotificationPriority.NORMAL,
                         source: str = "system", icon: str = None,
                         actions: List[Dict] = None, persistent: bool = False,
                         sound: bool = None, auto_dismiss: bool = True,
                         dismiss_delay: float = 5.0) -> str:
        """Bildirim göster"""
        
        # Bildirim ID oluştur
        notification_id = f"notif_{int(time.time() * 1000)}"
        
        # İkon belirleme
        if icon is None:
            icon_map = {
                NotificationType.INFO: "ℹ️",
                NotificationType.SUCCESS: "✅",
                NotificationType.WARNING: "⚠️",
                NotificationType.ERROR: "❌",
                NotificationType.SYSTEM: "⚙️",
                NotificationType.APP: "📱",
                NotificationType.SECURITY: "🔒",
                NotificationType.UPDATE: "🔄"
            }
            icon = icon_map.get(notification_type, "ℹ️")
        
        # Ses ayarı
        if sound is None:
            sound = self.sounds_enabled
        
        # Bildirim oluştur
        notification = NotificationData(
            id=notification_id,
            title=title,
            message=message,
            notification_type=notification_type,
            priority=priority,
            timestamp=datetime.now().isoformat(),
            source=source,
            icon=icon,
            actions=actions or [],
            persistent=persistent,
            sound=sound,
            auto_dismiss=auto_dismiss,
            dismiss_delay=dismiss_delay
        )
        
        # Kaydet
        self.notifications[notification_id] = notification
        self.notification_history.append(notification)
        
        # Geçmiş boyutunu kontrol et
        if len(self.notification_history) > self.max_history_size:
            self.notification_history = self.notification_history[-self.max_history_size:]
        
        # Kuyruğa ekle
        self.toast_queue.append(notification)
        
        self.logger.info(f"Notification queued: {title}")
        return notification_id
    
    def _process_queue(self):
        """Toast kuyruğunu işle"""
        if not PYQT_AVAILABLE:
            return
        
        # Maksimum toast sayısını kontrol et
        if len(self.active_toasts) >= self.max_visible_toasts:
            return
        
        # Kuyruktan al
        if not self.toast_queue:
            return
        
        # Önceliğe göre sırala
        self.toast_queue.sort(key=lambda n: self._get_priority_value(n.priority), reverse=True)
        
        notification = self.toast_queue.pop(0)
        self._show_toast(notification)
    
    def _get_priority_value(self, priority: NotificationPriority) -> int:
        """Öncelik değeri al"""
        values = {
            NotificationPriority.LOW: 1,
            NotificationPriority.NORMAL: 2,
            NotificationPriority.HIGH: 3,
            NotificationPriority.URGENT: 4
        }
        return values.get(priority, 2)
    
    def _show_toast(self, notification: NotificationData):
        """Toast göster"""
        if not PYQT_AVAILABLE:
            self.logger.warning("PyQt6 not available, cannot show toast")
            return
        
        try:
            # Toast oluştur
            toast = NotificationToast(notification)
            
            # Sinyalleri bağla
            toast.dismissed.connect(self._on_toast_dismissed)
            toast.action_clicked.connect(self._on_action_clicked)
            
            # Pozisyon hesapla
            self._position_toast(toast)
            
            # Göster
            toast.show_animated()
            
            # Aktif toast'lara ekle
            self.active_toasts[notification.id] = toast
            
            # Ses çal
            if notification.sound and self.sounds_enabled:
                self._play_notification_sound(notification.notification_type)
            
            # Callback tetikle
            self._trigger_callback("notification_shown", notification.to_dict())
            
            self.logger.info(f"Toast shown: {notification.title}")
            
        except Exception as e:
            self.logger.error(f"Failed to show toast: {e}")
    
    def _position_toast(self, toast: NotificationToast):
        """Toast pozisyonunu ayarla"""
        if not QApplication.instance():
            return
        
        screen = QApplication.primaryScreen()
        if not screen:
            return
        
        screen_geometry = screen.availableGeometry()
        toast_size = toast.size()
        
        # Mevcut toast'ların sayısını hesapla
        toast_index = len(self.active_toasts)
        spacing = 10
        
        if self.toast_position == "top-right":
            x = screen_geometry.width() - toast_size.width() - 20
            y = 20 + (toast_index * (toast_size.height() + spacing))
        elif self.toast_position == "top-left":
            x = 20
            y = 20 + (toast_index * (toast_size.height() + spacing))
        elif self.toast_position == "bottom-right":
            x = screen_geometry.width() - toast_size.width() - 20
            y = screen_geometry.height() - toast_size.height() - 20 - (toast_index * (toast_size.height() + spacing))
        elif self.toast_position == "bottom-left":
            x = 20
            y = screen_geometry.height() - toast_size.height() - 20 - (toast_index * (toast_size.height() + spacing))
        else:
            x = screen_geometry.width() - toast_size.width() - 20
            y = 20 + (toast_index * (toast_size.height() + spacing))
        
        toast.move(x, y)
    
    def _play_notification_sound(self, notification_type: NotificationType):
        """Bildirim sesi çal"""
        # TODO: Gerçek ses çalma implementasyonu
        self.logger.debug(f"Playing sound for {notification_type.value}")
    
    def _on_toast_dismissed(self, notification_id: str):
        """Toast kapatma olayı"""
        if notification_id in self.active_toasts:
            del self.active_toasts[notification_id]
            
            # Callback tetikle
            if notification_id in self.notifications:
                notification = self.notifications[notification_id]
                self._trigger_callback("notification_dismissed", notification.to_dict())
            
            self.logger.debug(f"Toast dismissed: {notification_id}")
    
    def _on_action_clicked(self, notification_id: str, action_id: str):
        """Aksiyon tıklama olayı"""
        self.logger.info(f"Action clicked: {action_id} on notification {notification_id}")
        
        # Callback tetikle
        if notification_id in self.notifications:
            notification = self.notifications[notification_id]
            self._trigger_callback("action_clicked", {
                "notification": notification.to_dict(),
                "action_id": action_id
            })
    
    def dismiss_notification(self, notification_id: str) -> bool:
        """Bildirimi kapat"""
        if notification_id in self.active_toasts:
            toast = self.active_toasts[notification_id]
            toast.dismiss()
            return True
        return False
    
    def dismiss_all_notifications(self):
        """Tüm bildirimleri kapat"""
        for notification_id in list(self.active_toasts.keys()):
            self.dismiss_notification(notification_id)
    
    def get_notification_history(self, limit: int = None) -> List[Dict]:
        """Bildirim geçmişini al"""
        history = self.notification_history
        if limit:
            history = history[-limit:]
        
        return [notification.to_dict() for notification in history]
    
    def clear_notification_history(self):
        """Bildirim geçmişini temizle"""
        self.notification_history.clear()
        self.logger.info("Notification history cleared")
    
    def get_active_notifications(self) -> List[Dict]:
        """Aktif bildirimleri al"""
        return [self.notifications[nid].to_dict() for nid in self.active_toasts.keys()]
    
    def set_settings(self, max_visible_toasts: int = None, sounds_enabled: bool = None,
                    animations_enabled: bool = None, toast_position: str = None):
        """Bildirim ayarlarını değiştir"""
        if max_visible_toasts is not None:
            self.max_visible_toasts = max_visible_toasts
        
        if sounds_enabled is not None:
            self.sounds_enabled = sounds_enabled
        
        if animations_enabled is not None:
            self.animations_enabled = animations_enabled
        
        if toast_position is not None:
            self.toast_position = toast_position
        
        self.logger.info("Notification settings updated")
    
    def add_callback(self, event_type: str, callback: Callable):
        """Callback ekle"""
        if event_type in self.callbacks:
            self.callbacks[event_type].append(callback)
    
    def remove_callback(self, event_type: str, callback: Callable):
        """Callback kaldır"""
        if event_type in self.callbacks:
            if callback in self.callbacks[event_type]:
                self.callbacks[event_type].remove(callback)
    
    def _trigger_callback(self, event_type: str, data: Any):
        """Callback tetikle"""
        for callback in self.callbacks.get(event_type, []):
            try:
                callback(data)
            except Exception as e:
                self.logger.error(f"Notification callback error for {event_type}: {e}")
    
    def get_notification_stats(self) -> Dict:
        """Bildirim istatistikleri"""
        try:
            # Türe göre sayım
            type_counts = {}
            for notification in self.notification_history:
                ntype = notification.notification_type.value
                type_counts[ntype] = type_counts.get(ntype, 0) + 1
            
            # Kaynak göre sayım
            source_counts = {}
            for notification in self.notification_history:
                source = notification.source
                source_counts[source] = source_counts.get(source, 0) + 1
            
            return {
                "total_notifications": len(self.notification_history),
                "active_notifications": len(self.active_toasts),
                "queued_notifications": len(self.toast_queue),
                "type_counts": type_counts,
                "source_counts": source_counts,
                "settings": {
                    "max_visible_toasts": self.max_visible_toasts,
                    "sounds_enabled": self.sounds_enabled,
                    "animations_enabled": self.animations_enabled,
                    "toast_position": self.toast_position
                }
            }
            
        except Exception as e:
            self.logger.error(f"Failed to generate notification stats: {e}")
            return {}
    
    def shutdown(self):
        """Modül kapatma"""
        self.logger.info("Shutting down notification manager...")
        
        # Timer'ı durdur
        if self.queue_timer:
            self.queue_timer.stop()
        
        # Tüm toast'ları kapat
        self.dismiss_all_notifications()
        
        # Kuyruğu temizle
        self.toast_queue.clear()
        
        self.logger.info("Notification manager shutdown completed")

# Kolaylık fonksiyonları
_notification_manager = None

def init_notifications(kernel=None):
    """Bildirim sistemini başlat"""
    global _notification_manager
    _notification_manager = NotificationManager(kernel)
    return _notification_manager

def show_info(title: str, message: str, **kwargs) -> str:
    """Bilgi bildirimi göster"""
    if _notification_manager:
        return _notification_manager.show_notification(
            title, message, NotificationType.INFO, **kwargs
        )
    return ""

def show_success(title: str, message: str, **kwargs) -> str:
    """Başarı bildirimi göster"""
    if _notification_manager:
        return _notification_manager.show_notification(
            title, message, NotificationType.SUCCESS, **kwargs
        )
    return ""

def show_warning(title: str, message: str, **kwargs) -> str:
    """Uyarı bildirimi göster"""
    if _notification_manager:
        return _notification_manager.show_notification(
            title, message, NotificationType.WARNING, **kwargs
        )
    return ""

def show_error(title: str, message: str, **kwargs) -> str:
    """Hata bildirimi göster"""
    if _notification_manager:
        return _notification_manager.show_notification(
            title, message, NotificationType.ERROR, **kwargs
        )
    return ""

def get_notification_manager() -> Optional[NotificationManager]:
    """Bildirim yöneticisini al"""
    return _notification_manager 


"""
Cloud Browser Core Modülleri
Modern PyCloud OS Web Browser
"""

from .browser_app import CloudBrowser
from .browser_ui import *
from .navigation import *
from .bookmarks import *
from .settings import *
from .pdf_viewer import *

__all__ = [
    'CloudBrowser'
] 